/* See LICENSE file for copyright and license details. */

/* Constants */
#define TERMINAL "st"
#define TERMCLASS "St"
#define BROWSER "librewolf"

/* appearance */
static unsigned int borderpx  = 1;        /* border pixel of windows */
static unsigned int snap      = 32;       /* snap pixel */
static int showbar            = 1;        /* 0 means no bar */
static int topbar             = 1;        /* 0 means bottom bar */
static char font[]            = "monospace:size=10";
static char dmenufont[]       = "monospace:size=10";
static const char *fonts[]          = { font };
static char normbgcolor[]           = "#000000";
static char normbordercolor[]       = "#000000";
static char normfgcolor[]           = "#BD93F9";
static char selfgcolor[]            = "#000000";
static char selbordercolor[]        = "#FF97C6";
static char selbgcolor[]            = "#BD93F9";
static char *colors[][3] = {
       /*               fg           bg           border   */
       [SchemeNorm] = { normfgcolor, normbgcolor, normbordercolor },
       [SchemeSel]  = { selfgcolor,  selbgcolor,  selbordercolor  },
};

typedef struct {
	const char *name;
	const void *cmd;
} Sp;
const char *spcmd1[] = {TERMINAL, "-n", "spterm", "-g", "120x34", NULL };
const char *spcmd2[] = {TERMINAL, "-n", "spcalc", "-f", "monospace:size=16", "-g", "50x20", "-e", "bc", "-lq", NULL };
static Sp scratchpads[] = {
	/* name          cmd  */
	{"spterm",      spcmd1},
	{"spcalc",    	spcmd2},
};

/* tagging */
static const char *tags[] = { "1", "2", "3", "4", "5", "6", "7", "8", "9" };
static const Rule rules[] = {
	/* xprop(1):
	 *	WM_CLASS(STRING) = instance, class
	 *	WM_NAME(STRING) = title
	 */
	/* class      instance    title       tags mask     isfloating   monitor */
	{ "FreeTube",   NULL,			NULL,		1 << 1,			0,			 -1 },
	{ "LibreWolf",  NULL,			NULL,		1 << 2,			0,			 -1 },
	{ "Beeper",     NULL,			NULL,		1 << 3,			0,			 -1 },
	{ NULL,		  "spterm",		NULL,		SPTAG(0),		1,			 -1 },
	{ NULL,		  "spcalc",		NULL,		SPTAG(1),		1,			 -1 },
};

/* layout(s) */
static float mfact     = 0.55; /* factor of master area size [0.05..0.95] */
static int nmaster     = 1;    /* number of clients in master area */
static int resizehints = 1;    /* 1 means respect size hints in tiled resizals */
static const int lockfullscreen = 1; /* 1 will force focus on the fullscreen window */

static const Layout layouts[] = {
	/* symbol     arrange function */
	{ "[]=",      tile },    /* first entry is default */
	{ "TTT",      bstack },
	{ "><>",      NULL },    /* no layout function means floating behavior */
};

/* key definitions */
#define MODKEY Mod4Mask
#define TAGKEYS(KEY,TAG) \
	{ MODKEY,                       KEY,      view,           {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask,           KEY,      toggleview,     {.ui = 1 << TAG} }, \
	{ MODKEY|ShiftMask,             KEY,      tag,            {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask|ShiftMask, KEY,      toggletag,      {.ui = 1 << TAG} },

/* helper for spawning shell commands in the pre dwm-5.0 fashion */
#define SHCMD(cmd) { .v = (const char*[]){ "/bin/sh", "-c", cmd, NULL } }

#define STATUSBAR "dwmblocks"

/* commands */
static char dmenumon[2] = "0"; /* component of dmenucmd, manipulated in spawn() */
static const char *dmenucmd[] = { "dmenu_run", "-m", dmenumon, "-fn", dmenufont, "-nb", normbgcolor, "-nf", normfgcolor, "-sb", selbordercolor, "-sf", selfgcolor, NULL };
static const char *termcmd[]  = { TERMINAL, NULL };

/*
 * Xresources preferences to load at startup
 */
ResourcePref resources[] = {
		{ "font",               STRING,  &font },
		{ "dmenufont",          STRING,  &dmenufont },
		{ "normbgcolor",        STRING,  &normbgcolor },
		{ "normbordercolor",    STRING,  &normbordercolor },
		{ "normfgcolor",        STRING,  &normfgcolor },
		{ "selbgcolor",         STRING,  &selbgcolor },
		{ "selbordercolor",     STRING,  &selbordercolor },
		{ "selfgcolor",         STRING,  &selfgcolor },
		{ "borderpx",          	INTEGER, &borderpx },
		{ "snap",          	INTEGER, &snap },
		{ "showbar",          	INTEGER, &showbar },
		{ "topbar",          	INTEGER, &topbar },
		{ "nmaster",          	INTEGER, &nmaster },
		{ "resizehints",       	INTEGER, &resizehints },
		{ "mfact",      	FLOAT,   &mfact },
};

static const Key keys[] = {
	/* modifier 			key 			function 			argument */
	{ MODKEY, 			XK_grave, 		spawn, 				{.v = (const char*[]){ "dmenuunicode", NULL } } },
	// { MODKEY|ShiftMask, 		XK_grave, 		togglescratch,			SHCMD("") },
	TAGKEYS( 			XK_1, 			0)
	TAGKEYS( 			XK_2, 			1)
	TAGKEYS( 			XK_3, 			2)
	TAGKEYS( 			XK_4, 			3)
	TAGKEYS( 			XK_5, 			4)
	TAGKEYS( 			XK_6, 			5)
	TAGKEYS( 			XK_7, 			6)
	TAGKEYS( 			XK_8, 			7)
	TAGKEYS( 			XK_9, 			8)
	// { MODKEY, 			XK_0, 			view, 				{.ui = ~0 } },
	{ MODKEY,     XK_j, 			focusstack, 			{.i = +1 } },
	{ MODKEY,     XK_k, 			focusstack, 			{.i = -1 } },
	// { MODKEY|ShiftMask, 		XK_0, 			tag, 				{.ui = ~0 } },
	{ MODKEY, 			XK_minus,		spawn, 				SHCMD("wpctl set-volume @DEFAULT_AUDIO_SINK@ 5%-; kill -44 $(pidof dwmblocks)") },
	{ MODKEY|ShiftMask, 		XK_minus, 		spawn, 				SHCMD("brightness -; pkill -RTMIN+27 dwmblocks") },
  // { MODKEY|ShiftMask,		          XK_minus,	spawn,		        SHCMD("wpctl set-volume @DEFAULT_AUDIO_SINK@ 15%-; kill -44 $(pidof dwmblocks)") },
	{ MODKEY,       XK_equal,       spawn,        SHCMD("wpctl set-volume @DEFAULT_AUDIO_SINK@ 5%+; kill -44 $(pidof dwmblocks)") },
	{ MODKEY|ShiftMask,		          XK_equal,	spawn,		        SHCMD("brightness +; pkill -RTMIN+27 dwmblocks") },
  // { MODKEY|ShiftMask,		          XK_equal,	spawn,		        SHCMD("wpctl set-volume @DEFAULT_AUDIO_SINK@ 15%+; kill -44 $(pidof dwmblocks)") },
	{ MODKEY,			                  XK_BackSpace,	spawn,		    {.v = (const char*[]){ "sysact", NULL } } },
	{ MODKEY|ShiftMask,		          XK_BackSpace,	spawn,		    {.v = (const char*[]){ "sysact", NULL } } },

	{ MODKEY,			                  XK_Tab,		view,		          {0} },
	// { MODKEY|ShiftMask,		      XK_Tab,		spawn,		        SHCMD("") },
	{ MODKEY,			                  XK_q,		  killclient,	      {0} },
	{ MODKEY|ShiftMask,		          XK_q,		  spawn,		        {.v = (const char*[]){ "sysact", NULL } } },
	{ MODKEY,			                  XK_w,		  spawn,		        {.v = (const char*[]){ BROWSER, NULL } } },
	// { MODKEY|ShiftMask,		          XK_w,		  spawn,		        {.v = (const char*[]){ TERMINAL, "-e", "sudo", "nmtui", NULL } } },
	{ MODKEY,			                  XK_e,		  spawn,		        SHCMD(TERMINAL " -e neomutt ; pkill -RTMIN+12 dwmblocks; rmdir ~/.abook") },
	// { MODKEY|ShiftMask,		          XK_e,		  spawn,		        SHCMD(TERMINAL " -e abook -C ~/.config/abook/abookrc --datafile ~/.config/abook/addressbook") },
	{ MODKEY,			                  XK_r,		  spawn,		        {.v = (const char*[]){ TERMINAL, "-e", "lfub", NULL } } },
	// { MODKEY|ShiftMask,		          XK_r,		  spawn,		        {.v = (const char*[]){ TERMINAL, "-e", "htop", NULL } } },
	{ MODKEY,			                  XK_t,		  setlayout,	      {.v = &layouts[0]} }, /* tile */
	{ MODKEY|ShiftMask,		          XK_t,		  setlayout,	      {.v = &layouts[1]} }, /* bstack */
	// { MODKEY,			                  XK_y,		  setlayout,	      {.v = &layouts[2]} }, /* spiral */
	// { MODKEY|ShiftMask,		          XK_y,		  setlayout,	      {.v = &layouts[3]} }, /* dwindle */
	// { MODKEY,			                  XK_u,		  setlayout,	      {.v = &layouts[4]} }, /* deck */
	// { MODKEY|ShiftMask,		          XK_u,		  setlayout,	      {.v = &layouts[5]} }, /* monocle */
	// { MODKEY,			                  XK_i,		  setlayout,	      {.v = &layouts[6]} }, /* centeredmaster */
	// { MODKEY|ShiftMask,		          XK_i,		  setlayout,	      {.v = &layouts[7]} }, /* centeredfloatingmaster */
	// { MODKEY,			                  XK_o,		  incnmaster,       {.i = +1 } },
	// { MODKEY|ShiftMask,		          XK_o,		  incnmaster,       {.i = -1 } },
	{ MODKEY,			                  XK_p,			spawn,		        {.v = (const char*[]){ "mpc", "toggle", NULL } } },
	{ MODKEY|ShiftMask,		          XK_p,			spawn,		        SHCMD("mpc stop; pauseallmpv") },
	{ MODKEY,			                  XK_bracketleft,		spawn,		{.v = (const char*[]){ "mpc", "seek", "-10", NULL } } },
	{ MODKEY|ShiftMask,		          XK_bracketleft,		spawn,		{.v = (const char*[]){ "mpc", "seek", "-60", NULL } } },
	{ MODKEY,			                  XK_bracketright,	spawn,		{.v = (const char*[]){ "mpc", "seek", "+10", NULL } } },
	{ MODKEY|ShiftMask,		          XK_bracketright,	spawn,		{.v = (const char*[]){ "mpc", "seek", "+60", NULL } } },
	{ MODKEY,			                  XK_backslash,		  view,		  {0} },
	// { MODKEY|ShiftMask,		      XK_backslash,		  spawn,		SHCMD("") },

	// { MODKEY,			                  XK_a,		          togglegaps,	{0} },
	// { MODKEY|ShiftMask,		          XK_a,		          defaultgaps,	{0} },
	// { MODKEY,			                  XK_s,		          togglesticky,	{0} },
	{ MODKEY|ShiftMask,		          XK_s,		          spawn,      {.v = (const char*[]){ "passmenu-otp", NULL } } },
	{ MODKEY,			                  XK_d,		          spawn,      {.v = (const char*[]){ "dmenu_run", NULL } } },
	{ MODKEY|ShiftMask,		          XK_d,		          spawn,		  {.v = (const char*[]){ "passmenu", NULL } } },
	{ MODKEY,			                  XK_f,		          togglefullscr,	{0} },
	{ MODKEY|ShiftMask,		          XK_f,		          setlayout,	{.v = &layouts[8]} },
	{ MODKEY,			                  XK_h,		          setmfact,	{.f = -0.05} },
	/* J and K are automatically bound above in STACKEYS */
	{ MODKEY,			                  XK_l,		          setmfact,      	{.f = +0.05} },
	{ MODKEY,			                  XK_apostrophe,	  togglescratch,	{.ui = 1} },
	// { MODKEY|ShiftMask,		      XK_apostrophe,	  spawn,		    SHCMD("") },
	// { MODKEY|ShiftMask,		          XK_apostrophe,	  togglesmartgaps,	{0} },
	{ MODKEY,			                  XK_Return,	      spawn,		{.v = termcmd } },
	{ MODKEY|ShiftMask,		          XK_Return,	      togglescratch,	{.ui = 0} },

	// { MODKEY,			                  XK_z,		          incrgaps,	{.i = +3 } },
	// { MODKEY|ShiftMask,		      XK_z,		          spawn,		SHCMD("") },
	// { MODKEY,			                  XK_x,		          incrgaps,	{.i = -3 } },
	// { MODKEY|ShiftMask,		      XK_x,		          spawn,		SHCMD("") },
	// { MODKEY,			                  XK_c,		          spawn,		{.v = (const char*[]){ TERMINAL, "-e", "profanity", NULL } } },
	// { MODKEY|ShiftMask,		      XK_c,		          spawn,		SHCMD("") },
	/* V is automatically bound above in STACKKEYS */
	// { MODKEY,			                  XK_b,		          togglebar,	{0} },
	// { MODKEY|ShiftMask,		      XK_b,		          spawn,		SHCMD("") },
	{ MODKEY,			                  XK_n,		          spawn,		{.v = (const char*[]){ TERMINAL, "-e", "nvim", "-c", "VimwikiIndex", NULL } } },
	{ MODKEY|ShiftMask,		          XK_n,		          spawn,		SHCMD(TERMINAL " -e newsboat ; pkill -RTMIN+6 dwmblocks") },
	{ MODKEY,			                  XK_m,		          spawn,		{.v = (const char*[]){ TERMINAL, "-e", "ncmpcpp", NULL } } },
	{ MODKEY|ShiftMask,		          XK_m,		          spawn,		SHCMD("wpctl set-mute @DEFAULT_AUDIO_SINK@ toggle; kill -44 $(pidof dwmblocks)") },
	{ MODKEY,			                  XK_comma,	        spawn,		{.v = (const char*[]){ "mpc", "prev", NULL } } },
	{ MODKEY|ShiftMask,		          XK_comma,	        spawn,		{.v = (const char*[]){ "mpc", "seek", "0%", NULL } } },
	{ MODKEY,			                  XK_period,	      spawn,		{.v = (const char*[]){ "mpc", "next", NULL } } },
	{ MODKEY|ShiftMask,		          XK_period,	      spawn,		{.v = (const char*[]){ "mpc", "repeat", NULL } } },

	// { MODKEY,			                  XK_Left,	        focusmon,	{.i = -1 } },
	// { MODKEY|ShiftMask,		          XK_Left,	        tagmon,		{.i = -1 } },
	// { MODKEY,			                  XK_Right,	        focusmon,	{.i = +1 } },
	// { MODKEY|ShiftMask,		          XK_Right,	        tagmon,		{.i = +1 } },

	{ MODKEY,			                  XK_Insert,	      spawn,		SHCMD("xdotool type $(grep -v '^#' ~/.local/share/larbs/snippets | dmenu -i -l 50 | cut -d' ' -f1)") },

	// { MODKEY,			                  XK_F1,		        spawn,		SHCMD("groff -mom /usr/local/share/dwm/larbs.mom -Tpdf | zathura -") },
	// { MODKEY,			                  XK_F2,		        spawn,		{.v = (const char*[]){ "tutorialvids", NULL } } },
	{ MODKEY,			                  XK_F3,		        spawn,		{.v = (const char*[]){ "displayselect", NULL } } },
	{ MODKEY,			                  XK_F4,		        spawn,		SHCMD(TERMINAL " -e pulsemixer; kill -44 $(pidof dwmblocks)") },
	// { MODKEY,			                  XK_F6,		        spawn,		{.v = (const char*[]){ "torwrap", NULL } } },
	// { MODKEY,			                  XK_F7,		        spawn,		{.v = (const char*[]){ "td-toggle", NULL } } },
	// { MODKEY,			                  XK_F8,		        spawn,		{.v = (const char*[]){ "mw", "-Y", NULL } } },
	{ MODKEY,			                  XK_F9,		        spawn,		{.v = (const char*[]){ "mounter", NULL } } },
	{ MODKEY,			                  XK_F10,		        spawn,		{.v = (const char*[]){ "unmounter", NULL } } },
	{ MODKEY,			                  XK_F11,		        spawn,		SHCMD("mpv --untimed --no-cache --no-osc --no-input-default-bindings --profile=low-latency --input-conf=/dev/null --title=webcam $(ls /dev/video[0,2,4,6,8] | tail -n 1)") },
	{ MODKEY,			                  XK_F12,		        spawn,		SHCMD("remaps") },
	{ MODKEY,			                  XK_space,	        zoom,		  {0} },
	// { MODKEY|ShiftMask,		          XK_space,	        togglefloating,	{0} },

	{ 0,				                    XK_Print,	        spawn,		SHCMD("maim pic-full-$(date '+%y%m%d-%H%M-%S').png") },
	{ ShiftMask,			              XK_Print,	        spawn,		{.v = (const char*[]){ "maimpick", NULL } } },
	{ MODKEY,			                  XK_Print,	        spawn,		{.v = (const char*[]){ "dmenurecord", NULL } } },
	{ MODKEY|ShiftMask,		          XK_Print,	        spawn,		{.v = (const char*[]){ "dmenurecord", "kill", NULL } } },
	{ MODKEY,			                  XK_Delete,	      spawn,		{.v = (const char*[]){ "dmenurecord", "kill", NULL } } },
	{ MODKEY,			                  XK_Scroll_Lock,	  spawn,		SHCMD("killall screenkey || screenkey &") },

	// { 0, XF86XK_AudioMute,		      spawn,		        SHCMD("wpctl set-mute @DEFAULT_AUDIO_SINK@ toggle; kill -44 $(pidof dwmblocks)") },
	// { 0, XF86XK_AudioRaiseVolume,	  spawn,		        SHCMD("wpctl set-volume @DEFAULT_AUDIO_SINK@ 3%+; kill -44 $(pidof dwmblocks)") },
	// { 0, XF86XK_AudioLowerVolume,	  spawn,		        SHCMD("wpctl set-volume @DEFAULT_AUDIO_SINK@ 3%-; kill -44 $(pidof dwmblocks)") },
	// { 0, XF86XK_AudioPrev,		      spawn,		        {.v = (const char*[]){ "mpc", "prev", NULL } } },
	// { 0, XF86XK_AudioNext,		      spawn,		        {.v = (const char*[]){ "mpc",  "next", NULL } } },
	// { 0, XF86XK_AudioPause,		      spawn,		        {.v = (const char*[]){ "mpc", "pause", NULL } } },
	// { 0, XF86XK_AudioPlay,		      spawn,		        {.v = (const char*[]){ "mpc", "play", NULL } } },
	// { 0, XF86XK_AudioStop,		      spawn,		        {.v = (const char*[]){ "mpc", "stop", NULL } } },
	// { 0, XF86XK_AudioRewind,	      spawn,		        {.v = (const char*[]){ "mpc", "seek", "-10", NULL } } },
	// { 0, XF86XK_AudioForward,	      spawn,		        {.v = (const char*[]){ "mpc", "seek", "+10", NULL } } },
	// { 0, XF86XK_AudioMedia,		      spawn,		        {.v = (const char*[]){ TERMINAL, "-e", "ncmpcpp", NULL } } },
	// { 0, XF86XK_AudioMicMute,	      spawn,		        SHCMD("pactl set-source-mute @DEFAULT_SOURCE@ toggle") },
	// /* { 0, XF86XK_PowerOff,	    	spawn,		        {.v = (const char*[]){ "sysact", NULL } } }, */
	// { 0, XF86XK_Calculator,		      spawn,		        {.v = (const char*[]){ TERMINAL, "-e", "bc", "-l", NULL } } },
	// { 0, XF86XK_Sleep,		          spawn,		        {.v = (const char*[]){ "sudo", "-A", "zzz", NULL } } },
	// { 0, XF86XK_WWW,		            spawn,		        {.v = (const char*[]){ BROWSER, NULL } } },
	// { 0, XF86XK_DOS,		            spawn,		        {.v = termcmd } },
	// { 0, XF86XK_ScreenSaver,	      spawn,		        SHCMD("slock & xset dpms force off; mpc pause; pauseallmpv") },
	// { 0, XF86XK_TaskPane,		        spawn,		        {.v = (const char*[]){ TERMINAL, "-e", "htop", NULL } } },
	// { 0, XF86XK_Mail,		            spawn,		        SHCMD(TERMINAL " -e neomutt ; pkill -RTMIN+12 dwmblocks") },
	// { 0, XF86XK_MyComputer,		      spawn,		        {.v = (const char*[]){ TERMINAL, "-e",  "lfub",  "/", NULL } } },
	// /* { 0, XF86XK_Battery,		      spawn,		        SHCMD("") }, */
	// { 0, XF86XK_Launch1,		        spawn,		        {.v = (const char*[]){ "xset", "dpms", "force", "off", NULL } } },
	// { 0, XF86XK_TouchpadToggle,	    spawn,		        SHCMD("(synclient | grep 'TouchpadOff.*1' && synclient TouchpadOff=0) || synclient TouchpadOff=1") },
	// { 0, XF86XK_TouchpadOff,	      spawn,		        {.v = (const char*[]){ "synclient", "TouchpadOff=1", NULL } } },
	// { 0, XF86XK_TouchpadOn,		      spawn,		        {.v = (const char*[]){ "synclient", "TouchpadOff=0", NULL } } },
	// { 0, XF86XK_MonBrightnessUp,	  spawn,		        {.v = (const char*[]){ "xbacklight", "-inc", "15", NULL } } },
	// { 0, XF86XK_MonBrightnessDown,	spawn,		        {.v = (const char*[]){ "xbacklight", "-dec", "15", NULL } } },

};

/* button definitions */
/* click can be ClkTagBar, ClkLtSymbol, ClkStatusText, ClkWinTitle, ClkClientWin, or ClkRootWin */
static const Button buttons[] = {
	/* click                event mask      button          function        argument */
	{ ClkLtSymbol,          0,              Button1,        setlayout,      {0} },
	{ ClkLtSymbol,          0,              Button3,        setlayout,      {.v = &layouts[2]} },
	{ ClkWinTitle,          0,              Button2,        zoom,           {0} },
	{ ClkStatusText,        0,              Button1,        sigstatusbar,   {.i = 1} },
	{ ClkStatusText,        0,              Button2,        sigstatusbar,   {.i = 2} },
	{ ClkStatusText,        0,              Button3,        sigstatusbar,   {.i = 3} },
	{ ClkClientWin,         MODKEY,         Button1,        movemouse,      {0} },
	{ ClkClientWin,         MODKEY,         Button2,        togglefloating, {0} },
	{ ClkClientWin,         MODKEY,         Button1,        resizemouse,    {0} },
	{ ClkTagBar,            0,              Button1,        view,           {0} },
	{ ClkTagBar,            0,              Button3,        toggleview,     {0} },
	{ ClkTagBar,            MODKEY,         Button1,        tag,            {0} },
	{ ClkTagBar,            MODKEY,         Button3,        toggletag,      {0} },
};

